@@ -7,7 +7,7 @@ use Carp;
 
 our $VERSION;
 BEGIN {
-    $VERSION = '0.29_1';
+    $VERSION = '0.14';
 }
 
 # guard against memory errors not being reported
@@ -26,10 +26,9 @@ our @ISA = qw(Exporter);
 sub CL_RAW            () { CL_SCAN_RAW() }
 sub CL_ARCHIVE        () { CL_SCAN_ARCHIVE() }
 sub CL_MAIL           () { CL_SCAN_MAIL() }
+sub CL_DISABLERAR     () { CL_SCAN_DISABLERAR() }
 sub CL_OLE2           () { CL_SCAN_OLE2() }
 sub CL_ENCRYPTED      () { CL_SCAN_BLOCKENCRYPTED() }
-sub CL_BLOCKENCRYPTED () { CL_SCAN_BLOCKENCRYPTED() }
-sub CL_MAILURL        () { CL_SCAN_MAILURL() }
 sub CL_HTML           () { CL_SCAN_HTML() }
 sub CL_PE             () { CL_SCAN_PE() }
 sub CL_BLOCKBROKEN    () { CL_SCAN_BLOCKBROKEN() }
@@ -38,33 +37,44 @@ sub CL_BLOCKMAX       () { CL_SCAN_BLOCKMAX() }
 # This allows declaration   use Mail::ClamAV ':all';
 # If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
 # will save memory.
-our %EXPORT_TAGS = ( all => [ qw(
+our %EXPORT_TAGS = ( 'all' => [ qw(
     retdbdir
 
     CL_CLEAN
     CL_VIRUS
-    CL_BREAK
 
     CL_EMAXREC
     CL_EMAXSIZE
     CL_EMAXFILES
+    CL_ERAR
+    CL_EZIP
+    CL_EMALFZIP
+    CL_EGZIP
+    CL_EBZIP
+    CL_EOLE2
+    CL_EMSCOMP
+    CL_EMSCAB
     CL_EACCES
     CL_ENULLARG
+
     CL_ETMPFILE
+    CL_EFSYNC
     CL_EMEM
     CL_EOPEN
     CL_EMALFDB
+    CL_EPATSHORT
     CL_ETMPDIR
     CL_ECVD
+    CL_ECVDEXTR
+    CL_EMD5
+    CL_EDSIG
+    CL_EIO
     CL_EFORMAT
 
-    CL_ENCINIT
-    CL_ENCLOAD
-    CL_ENCIO
-
     CL_SCAN_RAW
     CL_SCAN_ARCHIVE
     CL_SCAN_MAIL
+    CL_SCAN_DISABLERAR
     CL_SCAN_OLE2
     CL_SCAN_BLOCKENCRYPTED
     CL_SCAN_HTML
@@ -72,17 +82,12 @@ our %EXPORT_TAGS = ( all => [ qw(
     CL_SCAN_BLOCKBROKEN
     CL_SCAN_MAILURL
     CL_SCAN_BLOCKMAX
-    CL_SCAN_ALGORITHMIC
-    CL_SCAN_PHISHING_BLOCKSSL
-
-    CL_SCAN_PHISHING_BLOCKCLOAK
-    CL_SCAN_ELF
-
     CL_SCAN_STDOPT
 
     CL_RAW
     CL_ARCHIVE
     CL_MAIL
+    CL_DISABLERAR
     CL_OLE2
     CL_BLOCKENCRYPTED
     CL_HTML
@@ -90,6 +95,10 @@ our %EXPORT_TAGS = ( all => [ qw(
     CL_BLOCKBROKEN
     CL_MAILURL
     CL_BLOCKMAX
+
+
+    CL_VIRUS
+    CL_CLEAN
 ) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
@@ -149,6 +158,30 @@ sub scan {
     return $status;
 }
 
+sub scanbuff {
+    my $self = shift;
+
+    my $buff = shift;
+    croak "No buffer defined" unless defined $buff;
+
+    croak "Invalid arguments to scanbuff: @_" if @_;
+
+    my $st = _scanbuff($self, $buff);
+    my $status = new Mail::ClamAV::Status;
+    $status->error($st);
+    $status->errno(0+$st);
+    $status->clean($st == CL_CLEAN());
+    $status->virus($st == CL_VIRUS());
+    if ($status) {
+        $status->count(1);
+    }
+    else {
+        $status->count(0);
+    }
+    return $status;
+}
+
+
 use Inline C => Config =>
     VERSION  => $VERSION,
     PREFIX   => 'clamav_perl_',
@@ -168,26 +201,13 @@ use Inline C => <<'END_OF_C';
 #include <unistd.h>
 #include <clamav.h>
 
-
-#ifndef CL_ENCINIT
-#define CL_ENCINIT 0
-#endif
-#ifndef CL_ENCLOAD
-#define CL_ENCLOAD 0
-#endif
-#ifndef CL_ENCIO
-#define CL_ENCIO 0
-#endif
-#ifndef CL_DB_NCORE
-#define CL_DB_NCORE 0
-#endif
-
 #define SvClam(MEM) ((struct clamav_perl *)SvIV(SvRV(MEM)))
 
 static void error(int errcode);
 
 struct clamav_perl {
-    struct cl_engine *root;
+    struct cl_node *root;
+    struct cl_limits limits;
     struct cl_stat st;
     char is_dir;
     char *path;
@@ -205,21 +225,25 @@ SV *clamav_perl_new(char *class, char *path)
     if (stat(path, &st) != 0)
         croak("%s does not exist: %s\n", path, strerror(errno));
 
-    if ((status = cl_init(CL_INIT_DEFAULT)) != CL_SUCCESS) {
-        error(status);
-        return &PL_sv_undef;
-    }
-    if (!(c->root = cl_engine_new())) {
-        error(status);
-        return &PL_sv_undef;
-    }
-
     /* set defaults for limits */
-    cl_engine_set_num(c->root, CL_ENGINE_MAX_RECURSION, 5);
-    cl_engine_set_num(c->root, CL_ENGINE_MAX_FILES, 1000);
-    cl_engine_set_num(c->root, CL_ENGINE_MAX_FILESIZE, 1024 * 1028 * 10); /* 10 Megs */
+    c->limits.maxreclevel = 5;
+    c->limits.maxfiles = 1000;
+    c->limits.maxfilesize = 1024 * 1028 * 10; /* 10 Megs */
 
-    status = cl_load(path, c->root, &c->signatures, CL_DB_STDOPT);
+    /* XXX need to figure out a nice default */
+    c->limits.maxratio = 200;
+    c->limits.archivememlim = 1;
+
+    if (S_ISDIR(st.st_mode)) {
+        c->is_dir = 1;
+        memset(&c->st, 0, sizeof(struct cl_stat));
+        status = cl_statinidir(path, &c->st);
+        c->path = strdup(path);
+        if (status == 0)
+            status = cl_loaddbdir(path, &c->root, &c->signatures);
+    }
+    else
+        status = cl_loaddb(path, &c->root, &c->signatures);
 
     if (status != 0) {
         error(status);
@@ -254,107 +278,112 @@ char *clamav_perl_retdbdir()
 void clamav_perl_buildtrie(SV *self)
 {
     struct clamav_perl *c = SvClam(self);
-    cl_engine_compile(c->root);
-}
-
-int clamav_perl_build(SV *self)
-{
-    struct clamav_perl *c = SvClam(self);
-    int status;
-    status = cl_engine_compile(c->root);
-    if (status != CL_SUCCESS) {
-        cl_engine_free(c->root);
-        error(status);
-        return 0;
-    }
-    return 1;
+    cl_buildtrie(c->root);
 }
 
 int clamav_perl_maxreclevel(SV *self, ...)
 {
     Inline_Stack_Vars;
-    struct clamav_perl *c = SvClam(self);
-    int err = 0;
-    long long reclevel = 0;
     if (Inline_Stack_Items > 1) {
         SV *max;
         if (Inline_Stack_Items > 2)
             croak("Invalid number of arguments to maxreclevel()");
         max = Inline_Stack_Item(1);
-        cl_engine_set_num(c->root, CL_ENGINE_MAX_RECURSION, SvIV(max));
+        SvClam(self)->limits.maxreclevel = SvIV(max);
     }
-    reclevel = cl_engine_get_num(c->root, CL_ENGINE_MAX_RECURSION, &err);
-    if (err != CL_SUCCESS) {
-        error(err);
-        return;
-    }
-    return reclevel;
-}
-
-int clamav_perl_maxmailrec(SV *self, ...)
-{
-    warn("maxmailrec is not longer supported in clamav");
-    return 0;
+    return SvClam(self)->limits.maxreclevel;
 }
 
 int clamav_perl_maxfiles(SV *self, ...)
 {
     Inline_Stack_Vars;
-    struct clamav_perl *c = SvClam(self);
-    int err = 0;
-    long long maxfiles = 0;
     if (Inline_Stack_Items > 1) {
         SV *max;
         if (Inline_Stack_Items > 2)
             croak("Invalid number of arguments to maxfiles()");
         max = Inline_Stack_Item(1);
-        cl_engine_set_num(c->root, CL_ENGINE_MAX_FILES, SvIV(max));
+        SvClam(self)->limits.maxfiles = SvIV(max);
     }
-    maxfiles = cl_engine_get_num(c->root, CL_ENGINE_MAX_FILES, &err);
-    if (err != CL_SUCCESS) {
-        error(err);
-        return;
-    }
-    return maxfiles;
+    return SvClam(self)->limits.maxfiles;
 }
 
 int clamav_perl_maxfilesize(SV *self, ...)
 {
     Inline_Stack_Vars;
-    struct clamav_perl *c = SvClam(self);
-    int err = 0;
-    long long maxfsize;
     if (Inline_Stack_Items > 1) {
         SV *max;
         if (Inline_Stack_Items > 2)
             croak("Invalid number of arguments to maxfilesize()");
         max = Inline_Stack_Item(1);
-        cl_engine_set_num(c->root, CL_ENGINE_MAX_FILESIZE, SvIV(max));
+        SvClam(self)->limits.maxfilesize = SvIV(max);
     }
-    maxfsize = cl_engine_get_num(c->root, CL_ENGINE_MAX_FILESIZE, &err);
-    if (err != CL_SUCCESS) {
-        error(err);
-        return;
-    }
-    return maxfsize;
+    return SvClam(self)->limits.maxfilesize;
 }
 
 int clamav_perl_maxratio(SV *self, ...)
 {
-    warn("maxratio is not longer supported in clamav");
-    return 0;
+    Inline_Stack_Vars;
+    if (Inline_Stack_Items > 1) {
+        SV *max;
+        if (Inline_Stack_Items > 2)
+            croak("Invalid number of arguments to maxratio()");
+        max = Inline_Stack_Item(1);
+        SvClam(self)->limits.maxratio = (long int)SvIV(max);
+    }
+    return SvClam(self)->limits.maxratio;
 }
 
 int clamav_perl_archivememlim(SV *self, ...)
 {
-    warn("archivememlim is not longer supported in clamav");
-    return 0;
+    Inline_Stack_Vars;
+    if (Inline_Stack_Items > 1) {
+        SV *max;
+        if (Inline_Stack_Items > 2)
+            croak("Invalid number of arguments to archivememlim()");
+        max = Inline_Stack_Item(1);
+        SvClam(self)->limits.archivememlim = (short)SvIV(max);
+    }
+    return SvClam(self)->limits.archivememlim;
+}
+
+void clamav_perl__scanbuff(SV *self, SV *buff)
+{
+    struct clamav_perl *c = SvClam(self);
+    STRLEN len;
+    int status;
+    char *b;
+    const char *msg;
+    SV *smsg;
+    Inline_Stack_Vars;
+
+    Inline_Stack_Reset;
+
+    if (SvTAINTED(buff))
+        croak("buff argument specified to scanbuff() is tainted");
+
+    b = SvPV(buff, len);
+    status = cl_scanbuff(b, len, &msg, c->root);
+
+    smsg = sv_newmortal();
+    sv_setiv(smsg, (IV)status);
+
+    /* msg is some random memory if no virus was found */
+    if (status == CL_VIRUS)
+        sv_setpv(smsg, msg);
+    else if (status == CL_CLEAN)
+        sv_setpv(smsg, "Clean");
+    else
+        sv_setpv(smsg, cl_perror(status));
+
+    SvIOK_on(smsg);
+    Inline_Stack_Push(smsg);
+    Inline_Stack_Done;
 }
 
 void DESTROY(SV *self)
 {
     struct clamav_perl *c = SvClam(self);
-    cl_engine_free(c->root);
+    cl_freetrie(c->root);
     if (c->is_dir == 1)
         cl_statfree(&c->st);
     Safefree(c->path);
@@ -367,27 +396,28 @@ void clamav_perl__scanfd(SV *self, int fd, int options)
     STRLEN len;
     int status;
     unsigned long int scanned;
-    const char *virname;
+    const char *msg;
     SV *smsg, *sscanned;
     Inline_Stack_Vars;
 
     Inline_Stack_Reset;
 
     scanned = 0;
-    status = cl_scandesc(fd, &virname, &scanned, c->root, options);
+    status = cl_scandesc(fd, &msg, &scanned, c->root,
+            &c->limits, options);
     if (scanned == 0)
         scanned = 1;
 
     smsg = sv_newmortal();
     sv_setiv(smsg, (IV)status);
 
-    /* virname is some random memory if no virus was found */
+    /* msg is some random memory if no virus was found */
     if (status == CL_VIRUS)
-        sv_setpv(smsg, virname);
+        sv_setpv(smsg, msg);
     else if (status == CL_CLEAN)
         sv_setpv(smsg, "Clean");
     else
-        sv_setpv(smsg, cl_strerror(status));
+        sv_setpv(smsg, cl_perror(status));
 
     SvIOK_on(smsg);
     Inline_Stack_Push(smsg);
@@ -414,7 +444,8 @@ void clamav_perl__scanfile(SV *self, SV *path, int options)
 
     scanned = 0;
     p = SvPV(path, PL_na);
-    status = cl_scanfile(p, &msg, &scanned, c->root, options);
+    status = cl_scanfile(p, &msg, &scanned, c->root,
+            &c->limits, options);
     if (scanned == 0)
         scanned = 1;
 
@@ -427,7 +458,7 @@ void clamav_perl__scanfile(SV *self, SV *path, int options)
     else if (status == CL_CLEAN)
         sv_setpv(smsg, "Clean");
     else
-        sv_setpv(smsg, cl_strerror(status));
+        sv_setpv(smsg, cl_perror(status));
 
     SvIOK_on(smsg);
     Inline_Stack_Push(smsg);
@@ -450,66 +481,48 @@ static void error(int errcode)
     SV *err = get_sv("Mail::ClamAV::Error", TRUE);
 
     sv_setiv(err, (IV)errcode);
-    e = cl_strerror(errcode);
+    e = cl_perror(errcode);
     sv_setpv(err, e);
     SvIOK_on(err);
 }
 
 int clamav_perl_constant(char *name)
 {
-
-/* libclamav specific */
     if (strEQ("CL_CLEAN", name)) return CL_CLEAN;
-    if (strEQ("CL_SUCCESS", name)) return CL_SUCCESS;
     if (strEQ("CL_VIRUS", name)) return CL_VIRUS;
-    if (strEQ("CL_ENULLARG", name)) return CL_ENULLARG;
-    if (strEQ("CL_EARG", name)) return CL_EARG;
-    if (strEQ("CL_EMALFDB", name)) return CL_EMALFDB;
-    if (strEQ("CL_ECVD", name)) return CL_ECVD;
-    if (strEQ("CL_EVERIFY", name)) return CL_EVERIFY;
-    if (strEQ("CL_EUNPACK", name)) return CL_EUNPACK;
 
-/* I/O and memory errors */
-    if (strEQ("CL_EOPEN", name)) return CL_EOPEN;
-    if (strEQ("CL_ECREAT", name)) return CL_ECREAT;
-    if (strEQ("CL_EUNLINK", name)) return CL_EUNLINK;
-    if (strEQ("CL_ESTAT", name)) return CL_ESTAT;
-    if (strEQ("CL_EREAD", name)) return CL_EREAD;
-    if (strEQ("CL_ESEEK", name)) return CL_ESEEK;
-    if (strEQ("CL_EWRITE", name)) return CL_EWRITE;
-    if (strEQ("CL_EDUP", name)) return CL_EDUP;
-    if (strEQ("CL_EACCES", name)) return CL_EACCES;
-    if (strEQ("CL_ETMPFILE", name)) return CL_ETMPFILE;
-    if (strEQ("CL_ETMPDIR", name)) return CL_ETMPDIR;
-    if (strEQ("CL_EMAP", name)) return CL_EMAP;
-    if (strEQ("CL_EMEM", name)) return CL_EMEM;
-    if (strEQ("CL_ETIMEOUT", name)) return CL_ETIMEOUT;
-
-/* internal (not reported outside libclamav) */
-    if (strEQ("CL_BREAK", name)) return CL_BREAK;
     if (strEQ("CL_EMAXREC", name)) return CL_EMAXREC;
     if (strEQ("CL_EMAXSIZE", name)) return CL_EMAXSIZE;
     if (strEQ("CL_EMAXFILES", name)) return CL_EMAXFILES;
+    if (strEQ("CL_ERAR", name)) return CL_ERAR;
+    if (strEQ("CL_EZIP", name)) return CL_EZIP;
+    if (strEQ("CL_EMALFZIP", name)) return CL_EMALFZIP;
+    if (strEQ("CL_EGZIP", name)) return CL_EGZIP;
+    if (strEQ("CL_EBZIP", name)) return CL_EBZIP;
+    if (strEQ("CL_EOLE2", name)) return CL_EOLE2;
+    if (strEQ("CL_EMSCOMP", name)) return CL_EMSCOMP;
+    if (strEQ("CL_EMSCAB", name)) return CL_EMSCAB;
+    if (strEQ("CL_EACCES", name)) return CL_EACCES;
+    if (strEQ("CL_ENULLARG", name)) return CL_ENULLARG;
+
+    if (strEQ("CL_ETMPFILE", name)) return CL_ETMPFILE;
+    if (strEQ("CL_EFSYNC", name)) return CL_EFSYNC;
+    if (strEQ("CL_EMEM", name)) return CL_EMEM;
+    if (strEQ("CL_EOPEN", name)) return CL_EOPEN;
+    if (strEQ("CL_EMALFDB", name)) return CL_EMALFDB;
+    if (strEQ("CL_EPATSHORT", name)) return CL_EPATSHORT;
+    if (strEQ("CL_ETMPDIR", name)) return CL_ETMPDIR;
+    if (strEQ("CL_ECVD", name)) return CL_ECVD;
+    if (strEQ("CL_ECVDEXTR", name)) return CL_ECVDEXTR;
+    if (strEQ("CL_EMD5", name)) return CL_EMD5;
+    if (strEQ("CL_EDSIG", name)) return CL_EDSIG;
+    if (strEQ("CL_EIO", name)) return CL_EIO;
     if (strEQ("CL_EFORMAT", name)) return CL_EFORMAT;
 
-/* db options */
-    if (strEQ("CL_DB_PHISHING", name)) return CL_DB_PHISHING;
-    if (strEQ("CL_DB_PHISHING_URLS", name)) return CL_DB_PHISHING_URLS;
-    if (strEQ("CL_DB_PUA", name)) return CL_DB_PUA;
-    if (strEQ("CL_DB_CVDNOTMP", name)) return CL_DB_CVDNOTMP;
-    if (strEQ("CL_DB_OFFICIAL", name)) return CL_DB_OFFICIAL;
-    if (strEQ("CL_DB_PUA_MODE", name)) return CL_DB_PUA_MODE;
-    if (strEQ("CL_DB_PUA_INCLUDE", name)) return CL_DB_PUA_INCLUDE;
-    if (strEQ("CL_DB_PUA_EXCLUDE", name)) return CL_DB_PUA_EXCLUDE;
-    if (strEQ("CL_DB_COMPILED", name)) return CL_DB_COMPILED;
-
-/* recommended db settings */
-    if (strEQ("CL_DB_STDOPT", name)) return CL_DB_STDOPT;
-
-/* scan options */
     if (strEQ("CL_SCAN_RAW", name)) return CL_SCAN_RAW;
     if (strEQ("CL_SCAN_ARCHIVE", name)) return CL_SCAN_ARCHIVE;
     if (strEQ("CL_SCAN_MAIL", name)) return CL_SCAN_MAIL;
+    if (strEQ("CL_SCAN_DISABLERAR", name)) return CL_SCAN_DISABLERAR;
     if (strEQ("CL_SCAN_OLE2", name)) return CL_SCAN_OLE2;
     if (strEQ("CL_SCAN_BLOCKENCRYPTED", name)) return CL_SCAN_BLOCKENCRYPTED;
     if (strEQ("CL_SCAN_HTML", name)) return CL_SCAN_HTML;
@@ -517,36 +530,6 @@ int clamav_perl_constant(char *name)
     if (strEQ("CL_SCAN_BLOCKBROKEN", name)) return CL_SCAN_BLOCKBROKEN;
     if (strEQ("CL_SCAN_MAILURL", name)) return CL_SCAN_MAILURL;
     if (strEQ("CL_SCAN_BLOCKMAX", name)) return CL_SCAN_BLOCKMAX;
-    if (strEQ("CL_SCAN_ALGORITHMIC", name)) return CL_SCAN_ALGORITHMIC;
-    if (strEQ("CL_SCAN_PHISHING_BLOCKSSL", name)) return CL_SCAN_PHISHING_BLOCKSSL;
-    if (strEQ("CL_SCAN_PHISHING_BLOCKCLOAK", name)) return CL_SCAN_PHISHING_BLOCKCLOAK;
-    if (strEQ("CL_SCAN_ELF", name)) return CL_SCAN_ELF;
-    if (strEQ("CL_SCAN_PDF", name)) return CL_SCAN_PDF;
-    if (strEQ("CL_SCAN_STRUCTURED", name)) return CL_SCAN_STRUCTURED;
-    if (strEQ("CL_SCAN_STRUCTURED_SSN_NORMAL", name)) return CL_SCAN_STRUCTURED_SSN_NORMAL;
-    if (strEQ("CL_SCAN_STRUCTURED_SSN_STRIPPED", name)) return CL_SCAN_STRUCTURED_SSN_STRIPPED;
-    if (strEQ("CL_SCAN_PARTIAL_MESSAGE", name)) return CL_SCAN_PARTIAL_MESSAGE;
-    if (strEQ("CL_SCAN_HEURISTIC_PRECEDENCE", name)) return CL_SCAN_HEURISTIC_PRECEDENCE;
-
-    if (strEQ("CL_ENGINE_MAX_SCANSIZE", name)) return CL_ENGINE_MAX_SCANSIZE;
-    if (strEQ("CL_ENGINE_MAX_FILESIZE", name)) return CL_ENGINE_MAX_FILESIZE;
-    if (strEQ("CL_ENGINE_MAX_RECURSION", name)) return CL_ENGINE_MAX_RECURSION;
-    if (strEQ("CL_ENGINE_MAX_FILES", name)) return CL_ENGINE_MAX_FILES;
-    if (strEQ("CL_ENGINE_MIN_CC_COUNT", name)) return CL_ENGINE_MIN_CC_COUNT;
-    if (strEQ("CL_ENGINE_MIN_SSN_COUNT", name)) return CL_ENGINE_MIN_SSN_COUNT;
-    if (strEQ("CL_ENGINE_PUA_CATEGORIES", name)) return CL_ENGINE_PUA_CATEGORIES;
-    if (strEQ("CL_ENGINE_DB_OPTIONS", name)) return CL_ENGINE_DB_OPTIONS;
-    if (strEQ("CL_ENGINE_DB_VERSION", name)) return CL_ENGINE_DB_VERSION;
-    if (strEQ("CL_ENGINE_DB_TIME", name)) return CL_ENGINE_DB_TIME;
-    if (strEQ("CL_ENGINE_AC_ONLY", name)) return CL_ENGINE_AC_ONLY;
-    if (strEQ("CL_ENGINE_AC_MINDEPTH", name)) return CL_ENGINE_AC_MINDEPTH;
-    if (strEQ("CL_ENGINE_AC_MAXDEPTH", name)) return CL_ENGINE_AC_MAXDEPTH;
-    if (strEQ("CL_ENGINE_TMPDIR", name)) return CL_ENGINE_TMPDIR;
-    if (strEQ("CL_ENGINE_KEEPTMP", name)) return CL_ENGINE_KEEPTMP;
-
-    if (strEQ("CL_ENCINIT", name)) return CL_ENCINIT;
-    if (strEQ("CL_ENCLOAD", name)) return CL_ENCLOAD;
-    if (strEQ("CL_ENCIO", name)) return CL_ENCIO;
 
     if (strEQ("CL_SCAN_STDOPT", name)) return CL_SCAN_STDOPT;
 
@@ -606,18 +589,21 @@ Mail::ClamAV - Perl extension for the clamav virus scanner
         or die "Failed to load db: $Mail::ClamAV::Error";
 
     # When database is loaded, you must create the proper trie with:
-    $c->build or die "Failed to build engine: $Mail::ClamAV::Error";
+    $c->buildtrie;
 
     # check to see if we need to reload
     if ($c->statchkdir) {
         $c = new Mail::ClamAV(retdbdir());
-        $c->build or die "Failed to build engine: $Mail::ClamAV::Error";
+        $c->buildtrie;
     }
 
     # Set some limits (only applies to scan())
+    # Only relevant for archives
     $c->maxreclevel(4);
     $c->maxfiles(20);
     $c->maxfilesize(1024 * 1024 * 20); # 20 megs
+    $c->archivememlim(0); # limit memory usage for bzip2 (0/1)
+    $c->maxratio(0);
 
     # Scan a filehandle (scandesc in clamav)
     # scan(FileHandle or path, Bitfield of options)
@@ -652,78 +638,57 @@ Options for scanning.
 
 =over 1
 
-
 =item CL_SCAN_STDOPT
 
 This is an alias for a recommended set of scan options. You should use it to
-make your software ready for new features in the future versions of libclamav. 
+make your software ready for new features in future versions of libclamav.
 
 =item CL_SCAN_RAW
 
-Use it alone if you want to disable support for special files. 
+It does nothing. Please use it (alone) if you don't want to scan any special
+files.
 
 =item CL_SCAN_ARCHIVE
 
-This flag enables transparent scanning of various archive formats. 
+This flag enables transparent scanning of various archive formats.
 
 =item CL_SCAN_BLOCKENCRYPTED
 
-With this flag the library will mark encrypted archives as viruses
-(Encrypted.Zip, Encrypted.RAR). 
+With this flag the library marks encrypted archives as viruses (Encrypted.Zip,
+Encrypted.RAR).
 
 =item CL_SCAN_BLOCKMAX
 
 Mark archives as viruses if maxfiles, maxfilesize, or maxreclevel limit is
-reached. 
+reached.
 
 =item CL_SCAN_MAIL
 
-Enable support for mail files. 
+It enables support for mail files.
 
 =item CL_SCAN_MAILURL
 
 The mail scanner will download and scan URLs listed in a mail body. This flag
 should not be used on loaded servers. Due to potential problems please do not
-enable it by default but make it optional. 
+enable it by default but make it optional.
 
 =item CL_SCAN_OLE2
 
-Enables support for OLE2 containers (used by MS Office and .msi files). 
+Enables support for Microsoft Office document files.
 
 =item CL_SCAN_PE
 
-This flag enables deep scanning of Portable Executable files and allows
-libclamav to unpack executables compressed with run-time unpackers. 
-
-=item CL_SCAN_ELF
-
-Enable support for ELF files. 
+This flag enables scanning withing Portable Executable files and allows
+libclamav to unpack UPX, Petite, and FSG compressed executables.
 
 =item CL_SCAN_BLOCKBROKEN
 
 libclamav will try to detect broken executables and mark them as
-Broken.Executable. 
+Broken.Executable.
 
 =item CL_SCAN_HTML
 
-This flag enables HTML normalisation (including ScrEnc decryption). 
-
-=item CL_SCAN_ALGORITHMIC
-
-Enable algorithmic detection of viruses. 
-
-=item CL_SCAN_PHISHING_DOMAINLIST
-
-With a minor version bump clamav development team removed this and broke backwards compatibility,
-so it is no longer supported in this module as of 0.22.
-
-=item CL_SCAN_PHISHING_BLOCKSSL
-
-Phishing module: always block SSL mismatches in URLs. 
-
-=item CL_SCAN_PHISHING_BLOCKCLOAK
-
-Phishing module: always block cloaked URLs. 
+This flag enables HTML normalisation (including JScript decryption).
 
 =back
 
@@ -755,63 +720,55 @@ Error statuses
 
 =item CL_EMAXREC
 
-recursion limit exceeded 
+recursion level limit exceeded
 
 =item CL_EMAXSIZE
 
-size limit exceeded 
+size limit exceeded
 
 =item CL_EMAXFILES
 
-files limit exceeded 
+files limit exceeded
+
+=item CL_ERAR
+
+rar handler error
+
+=item CL_EZIP
+
+zip handler error
+
+=item CL_EMALFZIP
+
+malformed zip
+
+=item CL_EGZIP
+
+gzip handler error
+
+=item CL_EBZIP
+
+bzip2 handler error
+
+=item CL_EOLE2
+
+OLE2 handler error
+
+=item CL_EMSCOMP
+
+compress.exe handler error
+
+=item CL_EMSCAB 
+
+MS CAB module error
 
 =item CL_EACCES
 
-access denied 
+access denied
 
 =item CL_ENULLARG
 
-null argument 
-
-=item CL_ETMPFILE
-
-tmpfile() failed 
-
-=item CL_EMEM
-
-memory allocation error 
-
-=item CL_EOPEN
-
-file open error 
-
-=item CL_EMALFDB
-
-malformed database 
-
-=item CL_ETMPDIR
-
-mkdir() failed 
-
-=item CL_ECVD
-
-not a CVD file (or broken) 
-
-=item CL_EFORMAT
-
-bad format or broken file 
-
-=item CL_ENCINIT
-
-NodalCore initialization failed 
-
-=item CL_ENCLOAD
-
-error loading NodalCore database 
-
-=item CL_ENCIO
-
-general NodalCore I/O error 
+null argument error
 
 =back
 
@@ -840,12 +797,7 @@ These settings only apply to C<scan()> and archives (CL_SCAN_ARCHIVE).
 
 =item maxreclevel
 
-Sets the maximum recursion level into archives [default 5].
-
-=item maxmailrec
-
-With a minor version bump clamav development team removed this and broke backwards compatibility,
-so it is no longer supported in this module as of 0.22.
+Sets the maximum recursion level [default 5].
 
 =item maxfiles
 
@@ -859,8 +811,13 @@ disables the check.
 
 =item maxratio
 
-With a minor version bump clamav development team removed this and broke backwards compatibility,
-so it is no longer supported in this module as of 0.22.
+Maximum compression ratio. So if this is set to 200, libclamav will give up
+decompressing a file if it reaches 200x its compressed size [default 200]. A
+value of zero disables the check.
+
+=item archivememlim
+
+Turns on/off memory usage limits for bzip2. [default 1]
 
 =back
 
@@ -915,8 +872,16 @@ C<scan()>, it will C<croak()>.
 
 =item scanbuff($buff)
 
-This function was taken out of ClamAV in 0.90 and was thus taken out of
-Mail::ClamAV.
+scanbuff takes a raw buffer and scans it. No options are available for this
+function (it is assumed you already unarchived or de-MIMEed the buffer and that
+it is raw).
+
+NOTE: This method will go away in libclamav 0.90. Quote from the mailing list:
+
+    Please do not use cl_scanbuff at all, it's to be removed in 0.90. This
+    function only supports old type signature scanning and will miss many
+    viruses (just try to scan test/clam.exe). You should definitely use
+    cl_scanfile/cl_scandesc instead.
 
 =back
 
@@ -940,7 +905,7 @@ The ClamAV API documentation L<http://www.clamav.net/doc/html-0.65/node44.html>
 
 =head1 AUTHOR
 
-David P.C. Wollmann E<lt>converter42@gmail.comE<gt>
+Scott Beck E<lt>sbeck@gossamer-threads.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
